<?php
// $Id: settings.admin.inc,v 1.1.2.1 2009/07/04 18:12:23 overall Exp $

define('SETTINGS_FILE_EXT', 'modules_settings.txt');

function settings_get_modules_supporting_export_import() {
  $module_list = module_implements('settings_get_settings_info');
  foreach ($module_list as $key => $module) {
    $module_settings = settings_obtain_settings($module);
    if ($module_settings['multiple']) {
      unset($module_list[$key]);
    }
  }
  
  //$module_list = array_flip($module_list);
  if (!empty($module_list)) {
    $module_list = array_combine($module_list, $module_list);
  }
  return $module_list;
}

function settings_no_modules_supporting_export_import_form() {
  return array( 
    'modules' => array(
      '#type' => 'item',
      '#title' => t('Modules'),
      '#value' => t('There are no modules supporting settings export/import.'),
      '#weight' => 0,
    ),
  ); 
}

function settings_admin_options() {
  $form = array(
    'use_default_mark' => array(
      '#type' => 'textarea',
      '#title' => t('"Use Default" mark'), 
      '#description' => t('HTML code shown for settings form elements wich values are equal to default values.'),
      '#rows' => 1,
    ),
    'show_default_value' => array(
      '#type' => 'checkbox',
      '#title' => t('Show default value'), 
      '#description' => t('For settings form elements wich values are not equal to default values show its default values.'),
    ),
  );
  
  return system_settings_form(settings_prepare_settings_form_for('settings', $form)); 
}

function settings_admin_export() {
  $module_list = settings_get_modules_supporting_export_import();
  if (empty($module_list)) {
    return settings_no_modules_supporting_export_import_form();
  }
  
  return array(
    'modules' => array(
      '#type' => 'checkboxes',
      '#title' => t('Modules'),
      '#description' => t('Select modules to export settings.'),
      '#options' => $module_list,
      '#required' => TRUE,
      '#weight' => 0,
    ),
    'export' => array(
      '#type' => 'submit',
      '#value' => t('Export'),
      '#submit' => array('settings_export_file'),
      '#weight' => 1,
    ),
  ); 
}

function settings_admin_import() {
  $module_list = settings_get_modules_supporting_export_import();
  if (empty($module_list)) {
    return settings_no_modules_supporting_export_import_form();
  }
  
  return array(
    '#attributes' => array('enctype' => 'multipart/form-data'),
    'file' => array(
      '#type' => 'file',
      '#title' => t('Settings File'),
      '#description' => t('Select file with modules settings (@file_ext) to import settings.', array('@file_ext' => '*.'.SETTINGS_FILE_EXT)),
      '#weight' => 0,
    ),
    'import' => array(
      '#type' => 'submit',
      '#value' => t('Import'),
      '#submit' => array('settings_import_file'),
      '#weight' => 1,
    ),
  ); 
}

function settings_export_file($form, &$form_state) {
  $module_list = array();
  foreach ($form_state['values']['modules'] as $module => $selected) {
    if ($selected) {
      $module_list[] = $module;
    }  
  }
  
  $settings = array();
  foreach ($module_list as $module) {
    $settings[$module] = settings_get_all_for($module);
  }
  
  $file_name = join('_', $module_list).'.modules_settings.txt';
  
  drupal_set_header('Content-type: plain/text');
  drupal_set_header('Content-Disposition: attachment; filename="'.$file_name.'"');
  var_export($settings);
  exit;
}

function settings_import_file($form, &$form_state) {
  $module_list = settings_get_modules_supporting_export_import();
  
  $file_name = $_FILES['files']['tmp_name']['file'];
  $modules_settings = eval('return '.file_get_contents($file_name).';'); 
  
  foreach ($modules_settings as $module => $settings) {
    if (isset($module_list[$module])) {
      settings_set_all_for($module, $settings);
      drupal_set_message(t('!module module settings have been successfully imported.', array('!module' => '<b>'.$module.'</b>')));
    }
    else {
      drupal_set_message(t('Module !module is not installed or does not support settings import.', array('!module' => '<b>'.$module.'</b>')), 'warning');
    }
  }
}

?>